-- author: rafftnix
-- date: 11.12.2015


SilagePlate = {}

Vehicle.registerJointType("reckroewer");

function SilagePlate.prerequisitesPresent(specializations)
    return SpecializationUtil.hasSpecialization(Fillable, specializations);
end;

function SilagePlate:load(xmlFile)
	self.fillShovelFromTrigger = SilagePlate.fillShovelFromTrigger;
	self.getAllowFillShovel = SilagePlate.getAllowFillShovel;
	self.silagePlateRaycastCallback = SilagePlate.silagePlateRaycastCallback;
	
	self.silagePlateReckAttacherIndex = getXMLInt(xmlFile, "vehicle.silagePlate#reckAttacherJointIndex");
	self.silagePlateReckAttacherAnim = getXMLString(xmlFile, "vehicle.silagePlate#reckAttacherAnim");
	self.reckIsAttached = false;
	self.silagePlateRaycastNode = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.silagePlate#raycastNodeIndex"));
	self.shovelTipReferenceNode = self.silagePlateRaycastNode;
	self.allowFillFromShovelTrigger = true;
	self.detectedPlane = nil;
end;

function SilagePlate:delete()
  
end;

function SilagePlate:mouseEvent(posX, posY, isDown, isUp, button)
end;

function SilagePlate:keyEvent(unicode, sym, modifier, isDown)
end;

function SilagePlate:update(dt)
	
end;

function SilagePlate:updateTick(dt)
	if self.isServer and self:getIsActive() then
		self.detectedPlane = nil;
		local x, y, z = getWorldTranslation(self.silagePlateRaycastNode);
		local vx, vy, vz = 0, -1, 0;
		raycastAll(x, y, z, vx, vy, vz, "silagePlateRaycastCallback", 5, self);
		
		if self.fillLevel > 0 and ((self.movingDirection < 0 and self.lastSpeed*3600 > 3) or not self:isLowered(false)) then
			-- empty plate
			local delta = math.min(self.fillLevel, 15000/1000*dt);
			if self.detectedPlane ~= nil then
				local object = g_currentMission:getNodeObject(self.detectedPlane);
				if object ~= nil then
					if object.addShovelFillLevel ~= nil then
						delta = object:addShovelFillLevel(self, delta, self.currentFillType);
						self:setFillLevel(self.fillLevel - delta, self.currentFillType);
					elseif object.isHTP ~= nil then
						object:setFillLevel(object.fillLevel+delta, self.currentFillType);
						self:setFillLevel(self.fillLevel-delta, self.currentFillType);
					end;
				end;
			elseif g_currentMission.createNewHeapTipTrigger ~= nil then
				-- no bunker Silo found, so we create heap tip trigger
				if g_currentMission:canCreateHeapTipTrigger(self.components[1].node) then
					local x, y, z = getWorldTranslation(self.components[1].node);
					local yOffset = 1;
					local xRot = g_currentMission:getYRotHeapTipTrigger(self.components[1].node);
					local heapTipTrigger = g_currentMission:createNewHeapTipTrigger(x, yOffset, z, xRot);
					heapTipTrigger:setStatic();
					heapTipTrigger:setFillLevel(delta, self.currentFillType, true);
					self:setFillLevel(self.fillLevel-delta, self.currentFillType, true);
				end;
			else
				self:setFillLevel(0, self.currentFillType, false);
			end;
		elseif self:getAllowFillShovel(self.currentFillType) and self.lastSpeed*3600 > 1 then
			-- fill plate - filling from (heap tip trigger) shovel trigger is implemented somewhere else, so we only need to fill from bga silos
			if self.detectedPlane ~= nil then
				local object = g_currentMission:getNodeObject(self.detectedPlane);
				if object ~= nil and object.movingPlanes ~= nil then
					local delta = math.min((self.capacity-self.fillLevel), 10000/1000*dt);
					
					local closestPlaneIndex = 0;
					local closestPlaneDist = math.huge;
					local x, y, z = getWorldTranslation(self.components[1].node);
					for a=1, table.getn(object.movingPlanes) do
						local plane = object.movingPlanes[a];
						local wx, wy, wz = getWorldTranslation(plane.nodeId);
						local dist = math.sqrt((x-wx)*(x-wx) + (z-wz)*(z-wz));
						if dist < closestPlaneDist then
							closestPlaneIndex = a;
							closestPlaneDist = dist;
						end;						
					end;
					local plane = object.movingPlanes[closestPlaneIndex];
					delta = math.min(delta, plane.fillLevel);
					object:setMovingPlaneFillLevel(plane, plane.fillLevel - delta, false);
					
					local fillType = self.currentFillType;
					if fillType == 0 or fillType == Fillable.FILLTYPE_UNKNOWN then
						fillType = Fillable.fillTypeNameToInt["chaff"];
					end;
					self:setFillLevel(self.fillLevel+delta, fillType);
				end;
			end;
		end;
	end;
end;

function SilagePlate:draw()

end;

function SilagePlate:getAllowFillShovel(fillType)
	return self:allowFillType(self.currentFillType, false) and self.movingDirection > 0 and self:isLowered(false);
end;

-- usually used by shovels, here used by our plate to fill it.
-- the plate will be detected as shovel
function SilagePlate:fillShovelFromTrigger(shovelTrigger, deltaFillLevel, fillType, dt)
	if self:getAllowFillShovel(fillType) then
		deltaFillLevel = math.min(deltaFillLevel, self.capacity-self.fillLevel);
		self:setFillLevel(self.fillLevel+deltaFillLevel, fillType, false);
	
		return deltaFillLevel;
	else
		return 0;
	end;
end;

function SilagePlate:silagePlateRaycastCallback(hitObjectId, x, y, z, distance, nx, ny, nz, subShapeIndex)
	local object = g_currentMission:getNodeObject(hitObjectId);
	if object ~= nil then
		if object.addShovelFillLevel ~= nil and object.getAllowShovelFillType ~= nil then
			-- bunker silo
			if object:getAllowShovelFillType(self.currentFillType) or self.currentFillType == Fillable.FILLTYPE_UNKNOWN then
				if object.movingPlanes ~= nil then
					self.detectedPlane = hitObjectId;
				end;
			end;
		elseif object.getTipInfoForTrailer ~= nil and object.isHTP ~= nil and object.setFillLevel ~= nil then
			-- heap tip trigger
			if object:allowFillType(self.currentFillType, false) then
				self.detectedPlane = hitObjectId;
			end;
		end;
	end;
end;

function SilagePlate:attachImplement(implement)
	if self.silagePlateReckAttacherIndex ~= nil then
		if self.silagePlateReckAttacherIndex == implement.jointDescIndex then
			if self.isServer then
				self:setExtraWeights(0);
			end;
			self:playAnimation(self.silagePlateReckAttacherAnim, 1, nil, true);
			self.reckIsAttached = true;
		end;
	end;
end;

function SilagePlate:detachImplement(implementIndex, noEventSend)
	local implement = self.attachedImplements[implementIndex];
	if implement.object ~= nil then
		if self.silagePlateReckAttacherIndex ~= nil then
			if self.silagePlateReckAttacherIndex == implement.jointDescIndex then
				self:playAnimation(self.silagePlateReckAttacherAnim, -1, nil, true);
				self.reckIsAttached = false;
			end;
		end;
	end;
end;